/*
PostRedirect Handler is a HttpHandler to redirect POST request.
Copyright (C) 2005 COLIN Cyrille.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

using System;
using System.Text;
using System.Configuration;
using System.Collections.Specialized;
using System.Web;
using talweg.Utils;
using log4net;
using log4net.Config;

[assembly: log4net.Config.DOMConfigurator(Watch=true)]

namespace talweg.Handlers {
	class PostRedirectHandler: IHttpHandler {
		private static readonly log4net.ILog log = log4net.LogManager.GetLogger("PostRedirect");
		private static readonly string TalwegSuffix = ConfigurationSettings.AppSettings["TalwegSuffix"];
		public void ProcessRequest (HttpContext context) {
			try {
        Uri url = context.Request.Url;
        string prefix = (url.Scheme=="https")?"s--":"u--";
        string redirect_url = "https://" +
              Url.HostEncode(url.Host,TalwegSuffix,prefix) +
              url.PathAndQuery.Replace("/index.aspx","/");
				// the purpose is to send a 200 request with a page containing a form with redirect
				log.Info("POST Redirect url : "+ url.Host + url.PathAndQuery);
				context.Response.Write(GetForm(redirect_url,context.Request.Form));
			} catch (Exception e) {
				log.Error("POST ERREUR : " + e.ToString());
			}
		}
		public bool IsReusable {
			get { return true; }
		}
		private string GetForm(string action,NameValueCollection content) {
			StringBuilder sb = new StringBuilder();
			sb.Append("<html>\n<head>\n</head>\n");
			sb.Append("<script language=\"javascript\">\nfunction Submit() {\n\tthis.document.forms[0].submit();\n}\n</script>\n");
			sb.Append("<body onload=\"Submit()\">\n");
			sb.Append("<h1>The page you've requested have moved</h1>\n");
			sb.Append("<form action='"+action+"' method='POST' >\n");
			foreach(string key in content.AllKeys) {
				sb.Append("<input type=\"hidden\" name=\""+key+"\" value=\""+content[key].ToString()+"\" />\n");
			}
			sb.Append("<input type=\"submit\" name=\"Button\" value=\"Click Me\" id=\"Button\" />\n");
			sb.Append("</form>\n</body>\n</html>");
			return sb.ToString();
		}
	}
}

