/*
Talweg Handler is a HttpHandler to proxy requests.
Copyright (C) 2005 COLIN Cyrille.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

using System;
using System.IO;
using System.Web;
using System.Configuration;
using System.Web.SessionState;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Web.Security;
using ICSharpCode.SharpZipLib.GZip;
using ICSharpCode.SharpZipLib.Zip.Compression.Streams;
using System.Security.Cryptography.X509Certificates;
using talweg.Utils;
using log4net;
using log4net.Config;

[assembly: log4net.Config.DOMConfigurator(Watch=true)]

namespace talweg.Handlers {
	class TalwegHandler: IHttpHandler,IRequiresSessionState {
		private static readonly log4net.ILog log = log4net.LogManager.GetLogger("talweg");
		private static readonly log4net.ILog accesslog = log4net.LogManager.GetLogger("talweg-access");
	  private static readonly string TalwegSuffix = ConfigurationSettings.AppSettings["TalwegSuffix"];
	  private static readonly bool AcceptAllCertificates = (ConfigurationSettings.AppSettings["AcceptAllCertificates"]=="1")?true:false;
		private static long MaxContentLengthToParse = 100000; //100ko
		private static int RequestTimeOut = 1000000; //1000 seconds

		
		public void ProcessRequest (HttpContext context) {
			//
			//	Build the HttpWebRequest : req
			//
			HttpWebRequest req = null;
			HttpWebResponse response = null;
			CookieContainer cc = new CookieContainer();
			Uri url = context.Request.Url;
			string http_prefix = url.Host.StartsWith("s--")?"https://":"http://";
			
			if(Url.GetHost(url.Host,TalwegSuffix) == "bc.us.yahoo.com") http_prefix = "http://";
			
			// the Replace("/index.aspx,"/") is to correct the fact that mono add index.aspx on / terminated request
			string pathAndQuery = url.PathAndQuery.Replace("/index.aspx","/");
			Uri query_uri = new Uri(http_prefix + Url.GetHost(url.Host,TalwegSuffix)+pathAndQuery,true);
			//
			//	Access Log - [TODO] apply filter here instead of log4net filters for performance issue.
			//
			accesslog.Info(context.User.Identity.Name+" : "+query_uri.Scheme+"://"+query_uri.Host+query_uri.PathAndQuery);
			
			log.Info("Start HttpWebRequest : "+query_uri.Scheme+"://"+query_uri.Host+query_uri.PathAndQuery);
			req = (HttpWebRequest)WebRequest.Create(query_uri);
			req.AllowAutoRedirect = false;
			req.KeepAlive=false;
			req.ReadWriteTimeout = System.Threading.Timeout.Infinite;
			req.Timeout = System.Threading.Timeout.Infinite;
			//
			//  If need set proxy
			//
			if(context.Session["Proxy"] != null) {
				req.Proxy = (IWebProxy)context.Session["Proxy"];
			}
			//
			//	Attach cookie container, if a cookie container is attach with the session : using it else create a new one.
			//
			if(context.Session["Cookie-Container"] == null) {
				cc = new CookieContainer();
			} else {
				cc = (CookieContainer)context.Session["Cookie-Container"];
			}
			
			req.CookieContainer = new CookieContainer();
			foreach( Cookie c in cc.GetCookies(query_uri)) {
					if(!c.Expired)
								req.CookieContainer.Add(c);
			}
			log.Info("nb cookies to send : "+req.CookieContainer.Count.ToString());
			int cookieindex=1;
			foreach( Cookie c in req.CookieContainer.GetCookies(query_uri)) {
				log.Info(cookieindex.ToString()+" - name = "+c.Name+" Value  = "+c.Value + " expired =" + c.Expired.ToString()+ " expires =" + c.Expires.ToString());
				cookieindex++;
			}
			//
			// 	Build request header
			//
			BuildRequestHeader(req,context.Request);
			//force to gzip because i can't fix deflate pb !! [TODO]
			req.Headers.Set("Accept-Encoding","gzip");
			//
			//	If it's a POST request send post data.
			//
			log.Info("METHOD : "+ context.Request.HttpMethod);
			if(context.Request.HttpMethod == "POST") {
				req.Method = "POST";
				req.ContentType = context.Request.Headers.Get("Content-Type");
				req.ContentLength = context.Request.InputStream.Length;
				//
				// Expect to null : site like netvibes.com doesn't accept Except header set to 100-continue.
				// 
				System.Net.ServicePointManager.Expect100Continue = false;
				
				Stream req_stream = req.GetRequestStream();
				byte[] buf = new byte[4096];
	  			int l = 0;
				long content_length = 0;
    			while ((l =  context.Request.InputStream.Read(buf, 0, 4096)) > 0) {
     				req_stream.Write(buf, 0, l);
					content_length += l;
	     		}
				req_stream.Close();
				log.Info("Post : -" + req.ContentLength + "-" + req.ContentType);
			}
			try{
				// Trust all certificates	
				if(AcceptAllCertificates)System.Net.ServicePointManager.CertificatePolicy = new TrustAllCertificatesPolicy();
				//
				// 	Get HttpWebResponse : response
				//
				try {
			     	response = (HttpWebResponse)req.GetResponse ();
				} catch (WebException w){
						if(w.Response != null ) {
								response =(HttpWebResponse)w.Response;
								if( response.StatusCode != HttpStatusCode.NotFound ) {
										log.Error("WebException " + req.RequestUri + ":\n\t\t" + w.Message.ToString());
								} else {
										log.Debug("WebException " + req.RequestUri + ":\n\t\t" + w.Message.ToString());
								}
						} else {
								throw w;
						}
				}
				//
				//	Save Cookies
				//
				log.Info("Modify cookie");
				//cc.Add(response.Cookies);
				// Test to resolve document.domain for specifics Urls
				foreach(Cookie c in response.Cookies) {
						if(query_uri.Host.EndsWith("webmessenger.msn.com")) {
								c.Domain = ".msn.com";
						}
						cc.Add(c);
				}
				context.Session["Cookie-Container"] = cc;
				//	Send cookies to client : Modify = remove 'Expires' and Domain.
				foreach(string key in response.Headers.AllKeys) {
				  if( key == "Set-Cookie") {
							foreach(string s in response.Headers.GetValues(key)) {
					  			context.Response.AddHeader(key,ModifyCookie(s));
							}
					}
				}
				//
				//	Fill header All fields except Set-Cookie // Transfert-Encoding // Content-Encoding
				//
				BuildResponseHeader(response,context.Response);
				//
				//	Check Status code
				//
				context.Response.StatusCode = (int)response.StatusCode;
				if(	response.StatusCode == HttpStatusCode.Found ||
						response.StatusCode == HttpStatusCode.Redirect ||
						response.StatusCode == HttpStatusCode.Moved ||
						response.StatusCode == HttpStatusCode.MovedPermanently ) {
						
						//context.Response.AddHeader("Location", response.Headers["Location"].ToString());		
						context.Response.AddHeader("Location", Url.ParseUrl(response.Headers["Location"].ToString(),TalwegSuffix) );		
						//log.Info("Redirect received, new location : "+Url.ParseUrl(response.Headers["Location"].ToString(),TalwegSuffix) );
										
				} else if (	response.StatusCode == HttpStatusCode.NotFound ) {
						log.Info("STATUS CODE NOT FOUND : Terminate process");
						response.Close();
	      	  return;
								
				} else if (	response.StatusCode == HttpStatusCode.NotModified ) { 
								response.Close();
								return;
				} else if ( response.StatusCode == HttpStatusCode.Unauthorized ) {
								// Asp.net use 401 code to redirect to login page. So Code401Module  convert 901 to 401 at the end of request.
								context.Response.StatusCode = 901;
				}

				log.Info("STATUS CODE : "+ response.StatusCode.ToString());
				//
				//	For text/plain and text/html we retrieve the content to parse link.
				//	If there's a content type without charset we add a empty charset 
				//	to prevent that .Net fill it with a default value
				//
				if(response.ContentType == null) { response.Close(); return; }
				if(		(response.ContentType.IndexOf("text/plain") != -1 || 
							response.ContentType.IndexOf("text/html") != -1	|| 
							response.ContentType.IndexOf("application/x-javascript") != -1 )
							&& response.ContentLength < MaxContentLengthToParse	) {
					// 
					// PARSE STUFF
					// 
					// 1 - set default encoding
					// 2 - retrieve content
					// 3 - check for charset in header or in meta charset tag
					// 4 - if necessary change default encoding
					// 5 - Read content
					// 6 - Parse
					// 7 - Send Data in correct encoding
					//
					// Set default encoding to ISO-8859-1
					System.Text.Encoding encoding = System.Text.Encoding.GetEncoding("ISO-8859-1");
					// read page content
					log.Info("read page content");
					MemoryStream st = new MemoryStream();
					byte[] buf = new byte[4096];
					int l = 0;
					while ((l =  response.GetResponseStream().Read(buf, 0, 4096)) > 0) {
						st.Write(buf, 0, l);
					}
					log.Info("try to retrieve correct encoding");
					//	try to retrieve correct encoding
					Match m = Regex.Match(response.ContentType,"charset=(.*)");
					if(m.Groups.Count > 1) {
						try {
							encoding = Encoding.GetEncoding(m.Groups[1].ToString().Trim());
							log.Info("Change default character encoding : "+ encoding.ToString());
							context.Response.AddHeader("Content-Type",response.ContentType);
						} catch {}
					} else {	
						//
						// we must load the page and look meta 
						// if there'no response it could be a Iframe so we must know parent iframe encoding [TODO]
						// i think the best way is to create a hashtable of parsed url (enter here) .
						// 
						st.Position = 0;
						StreamReader tmp_reader = GetStreamReader(response,encoding,st);
						string temp_page = tmp_reader.ReadToEnd();
						Match temp_m = Regex.Match(temp_page,"<meta.*content-type.*>",RegexOptions.IgnoreCase | RegexOptions.Multiline);
						if(temp_m.Groups.Count > 0) {
							temp_m = Regex.Match(temp_m.Groups[0].ToString(),"charset=[\\\"\\s]?(.*?)[\\\"\\s]");
							if(temp_m.Groups.Count > 1) {
								try { // if error keep default encoding
									encoding = Encoding.GetEncoding(temp_m.Groups[1].ToString().Trim());
								} catch {}
								log.Info("Change default character encoding from page : "+ response.ContentType);
							}
						}
						//
						//	This is to prevent asp.Net to set charset to Utf-8 when charset is not set.
						context.Response.AddHeader("Content-Type",response.ContentType +"; charset=");
						//tmp_reader.Close();   // do not close reader else it's closed st !!
					}
					st.Position = 0;
					StreamReader reader = GetStreamReader(response,encoding,st);
					//string page = reader.ReadToEnd();
					//page = Parse(page);
					string page = String.Empty;
					page = ParseHtml(reader);
					//page = ParseHttp(reader);
					//page = ParseJS(new StreamReader(new MemoryStream(encoding.GetBytes(page)),encoding));
					//	Debug stuff : don't uncomment !
					//log.Info(page);
					//
					// [TODO]	look for accept Request header and encode page
					//
					st = new MemoryStream(encoding.GetBytes(page));
					context.Response.AddHeader("Content-Length",st.Length.ToString());
					buf = new byte[4096];
			        l = 0;
          			while ((l =  st.Read(buf, 0, 4096)) > 0) {
		            	context.Response.OutputStream.Write(buf, 0, l);
			         }
				}else{
					log.Info("Non parsed stuff");
					//
					//	here we copy received Content-Encoding, 
					//	this one hasn't been copied before because in parsage case we do not encode
					//
					if(response.Headers["Content-Encoding"] != null)
									context.Response.AddHeader("Content-Encoding",Join(response.Headers.GetValues("Content-Encoding")));
					//
					//	If there's content write it to the output stream
					//
					// Commented until path applied to mod_mono
					//bool isconnected = context.Response.IsClientConnected;
					//log.Info(context.Response.IsClientConnected?"Client connected":"client disconnected");
					Stream st = response.GetResponseStream();
					context.Response.Buffer = false;
					log.Info("Get Stream");
					byte[] buf = new byte[32*1024];
  				int l = 0;
	    		while ((l =  st.Read(buf, 0, 32*1024)) > 0) {
  					context.Response.OutputStream.Write(buf, 0, l);
	      	}
					st.Close();
				}
				
				//
				//	Finally Close() [TODO] Control all object that need to be closed
				//
				response.Close();
			}catch (WebException e){
				// TODO i think this is no need, webexception are catched above
				log.Error(e.ToString());
				if(response	!=null)	response.Close();
				// NameResolutionFailure
				throw new HttpException (500, "talweg, Server Error to request : " +req.RequestUri+"  "+e.ToString());
		  } catch (Exception e) {
				log.Error("An error wes occured when trying to retrieve this url : "+req.RequestUri+"\n"+e.ToString());
				throw new HttpException (500, "talweg, Server Error to request : " +req.RequestUri+"\n"+ e.Message);
			}
		}
		public bool IsReusable {
			get { return true; }
		}
		private StreamReader GetStreamReader(HttpWebResponse response,Encoding encoding,MemoryStream st) {
			StreamReader reader = null;
			if(response.ContentEncoding == "gzip") {
				reader = new StreamReader(new GZipInputStream(st),encoding);
      } else if (response.ContentEncoding == "deflate" ) {
        reader = new StreamReader(new InflaterInputStream(st),encoding);
      } else {
        reader = new StreamReader(st,encoding);
      }
			return reader;
		}
		private void BuildRequestHeader(HttpWebRequest req,HttpRequest request) {
			foreach(string key in request.Headers.AllKeys) {
				if(!WebHeaderCollection.IsRestricted(key) && key != "Cookie" )req.Headers[key] = request.Headers.Get(key);
			}
			// set protected header
			if(request.Headers.Get("Accept")!=null) req.Accept = request.Headers.Get("Accept");
			if(request.Headers.Get("Expect")!=null) req.Expect = request.Headers.Get("Expect");
			//if(request.Headers.Get("Referer")!=null) req.Referer = request.Headers.Get("Referer");
			if(request.Headers.Get("User-Agent")!=null) req.UserAgent = request.Headers.Get("User-Agent");
			if(request.Headers.Get("Referer")!=null) {
					req.Referer = Url.GetUrl(request.Headers.Get("Referer"),TalwegSuffix,true);
					log.Info("Change Referer : "+ req.Referer);
			}
		}
		private void BuildResponseHeader(HttpWebResponse res,HttpResponse response) {
			foreach(string key in res.Headers.AllKeys) {
				if(	String.Compare(key,"Transfer-Encoding",true) != 0 
					&& String.Compare(key,"Content-Encoding",true) != 0
					&& String.Compare(key,"Set-Cookie",true) !=0 ) {
					response.AddHeader(key,Join(res.Headers.GetValues(key)));
				} else {
					log.Info("Header not set : "+key+"--"+Join(res.Headers.GetValues(key)));
				}
			}
		}
		private string Join(string[] array) {
			string result = String.Empty;
			foreach (string s in array){
				result += ";"+s ;
			}
			if(result.StartsWith(";"))result = result.Substring(1);
			return result;
		}
		private string ModifyCookie(string cookie) {
				string[] values = cookie.Split(';');
				string result = String.Empty;
				foreach(string s in values) {
					//log.Info("cookie : "+s);
					if(s.Trim().ToLower().StartsWith("domain")) {
						result += "domain="+TalwegSuffix+";";
					} else if (s.Trim().ToLower().StartsWith("expires")){
						//do nothing = remove
						//result += s +";";
					} else {
						result += s +";";
					}
				}
				if(result.EndsWith(";"))result = result.Substring(0,result.LastIndexOf(";"));
				return result;
		}
		private string ParseHtml(StreamReader reader) {
			log.Info("Enter in html parser");
			StringBuilder result = new StringBuilder();
			string line = String.Empty;
			bool first = true;
			try {
			while((line = reader.ReadLine()) != null) {
				if(first){
					first=false;
				} else {
					result.Append("\n");
				}
				Match m = Regex.Match(line,"(codebase|background|action|src|href)(\\s*=\\s*[\\\"\\\']?\\s*)(http.*?)([\\\"\\\'\\s>])",RegexOptions.IgnoreCase | RegexOptions.Multiline);
	      while(m.Groups.Count > 1) {
						if(!(m.Groups[3].ToString().StartsWith("https://s--") 
								|| m.Groups[3].ToString().StartsWith("https://u--")))
		           line = line.Replace(	m.Groups[0].ToString(),m.Groups[1].ToString()+
															 			m.Groups[2].ToString()+Url.ParseUrl(m.Groups[3].ToString(),TalwegSuffix)+
																		m.Groups[4].ToString());
	          m = m.NextMatch();
	      }
  	    // webmessenger bug : need to parse document.domain;
    	  m = Regex.Match(line,"(document\\.domain=[\\\"\\\'].*?[\\\"\\\'])",RegexOptions.IgnoreCase | RegexOptions.Multiline);
				if (m.Groups.Count > 1) {
					string newdomain = TalwegSuffix;
					if(newdomain.StartsWith("."))
									newdomain = newdomain.Substring(1);
				  line = line.Replace(m.Groups[0].ToString(),"document.domain='"+newdomain+"'");
					log.Info("Change document.domain : "+m.Groups[0].ToString());
      	}
				result.Append(line);
			}
			}catch{}
			return result.ToString();
		}
		private string ParseHttp(StreamReader reader) {
	    log.Info("Enter in http:// parser");
      StringBuilder result = new StringBuilder();
      string line = String.Empty;
      while((line = reader.ReadLine()) != null) {
				Match m = Regex.Match(line,"(http.*?)([\\\"\\\'\\s>])",RegexOptions.IgnoreCase | RegexOptions.Multiline);
				while(m.Groups.Count > 1) {
        	if(!(m.Groups[3].ToString().StartsWith("https://s--") || m.Groups[3].ToString().StartsWith("https://u--")))
               line = line.Replace( m.Groups[0].ToString(),Url.ParseUrl(m.Groups[1].ToString(),TalwegSuffix)+ m.Groups[2].ToString());
            m = m.NextMatch();
        }
				result.Append(line+"\n");
			}
      return result.ToString();
		}			
    private string ParseJS(StreamReader reader) {
      log.Info("Enter in javascript parser");
      StringBuilder result = new StringBuilder();
      string line = String.Empty;
      while((line = reader.ReadLine()) != null) {
        Match m = Regex.Match(line,"(http[s]?://[^us][^-].*?)([\\\"'\\s;@])",RegexOptions.IgnoreCase | RegexOptions.Multiline);
        while(m.Groups.Count > 1) {
      	   log.Info("Found in javascript parser");
					 
           line = line.Replace(m.Groups[1].ToString(),Url.ParseUrl(m.Groups[1].ToString(),TalwegSuffix));
	         m = m.NextMatch();
        }
        // webmessenger bug : need to parse document.domain;
        m = Regex.Match(line,@"(document\.domain='.*?')",RegexOptions.IgnoreCase | RegexOptions.Multiline);
        if (m.Groups.Count > 1) {
           line = line.Replace(m.Groups[0].ToString(),"document.domain='"+TalwegSuffix+"'");
        }
        result.Append(line+"\n");
      }
      return result.ToString();
    }
	
	}
	public class TrustAllCertificatesPolicy : System.Net.ICertificatePolicy {
		public bool CheckValidationResult(ServicePoint sp, X509Certificate cert,WebRequest req, int problem){
			return true;
		}
	}
}

