/*
 * Talweg Handler is a HttpHandler to proxy requests.
 * Copyright (C) 2007 COLIN Cyrille.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * */

using System;
using talweg.Utils;
using NUnit.Framework;

namespace MonoTests.talweg.Utils {
	
		[TestFixture]
    public class UrlTest {
	
			[Test]
			public void TestParseUrl() {
				Assertion.AssertEquals ("ParseUrl01",
																"http://www.google.fr/",
																Url.GetUrl("https://u--www--google--fr.my.talweg/","my.talweg",true));
				Assertion.AssertEquals ("ParseUrl02",
																"http://www.google.fr",
																Url.GetUrl("https://u--www--google--fr.my.talweg","my.talweg",true));
				Assertion.AssertEquals ("ParseUrl03",
																"http://www.google.fr/test/test/test.html", 
																Url.GetUrl("https://u--www--google--fr.my.talweg/test/test/test.html","my.talweg",true));
    	}
			[Test]
			public void TestGetHost() {
				Assertion.AssertEquals ("GetHost01",
																"www.google.fr",
																Url.GetHost("u--www--google--fr.my.talweg","my.talweg"));
				Assertion.AssertEquals ("GetHost02",
																"www.google.test.fr",
																Url.GetHost("s--www--google--test--fr.my.talweg","my.talweg"));
				Assertion.AssertEquals ("GetHost03",
																"www.google.fr",
																Url.GetHost("u--www--google--fr.my.talweg","my.talweg"));
			}
			[Test]
			public void TestCleanUrl() {
				Assertion.AssertEquals ("CleanUrl01",
																"",
																Url.CleanUrl("",""));
				
				Assertion.AssertEquals ("CleanUrl02",
																"http://toolbarqueries.google.fr/search?client=navclient-auto&googleip=P;;0&iqrn=lyfC&orig=0CaVk&ie=UTF-8&oe=UTF-8&features=Rank:&q=info:http%3a%2f%2fwww%2etele2internet%2efr%2f&ch=76331114465",	
																Url.CleanUrl(
																"http://toolbarqueries.google.fr/search?client=navclient-auto&googleip=P;;0&iqrn=lyfC&orig=0CaVk&ie=UTF-8&oe=UTF-8&features=Rank:&q=info:https%3a%2f%2fu%2d%2dwww%2d%2dtele2internet%2d%2dfr%2ewifi%2euniv%2dmetz%2efr%2f&ch=76331114465","wifi.univ-metz.fr"));
				
				Assertion.AssertEquals ("CleanUrl03",
																"http://toolbarqueries.google.fr/search?client=navclient-auto&googleip=P;;0&iqrn=lyfC&orig=0CaVk&ie=UTF-8&oe=UTF-8&features=Rank:&q=info:https%3a%2f%2fu%2d%2dwww%2d%2dtele2internet%2d%2dfr%2eeuniv%2dmetz%2efr%2f&ch=76331114465",
																Url.CleanUrl(
																"http://toolbarqueries.google.fr/search?client=navclient-auto&googleip=P;;0&iqrn=lyfC&orig=0CaVk&ie=UTF-8&oe=UTF-8&features=Rank:&q=info:https%3a%2f%2fu%2d%2dwww%2d%2dtele2internet%2d%2dfr%2eeuniv%2dmetz%2efr%2f&ch=76331114465","wifi.univ-metz.fr"));
				
				Assertion.AssertEquals ("CleanUrl04",
																"http://g.msn.com/_0USHP/32?http://fr.msn.com/&&ps=10225&pi=9484&di=81&gt1=&ce=2&cm=wrapper%3Etickler&hl=Search%20Prompt%20shown&su=http://fr.msn.com/",
																Url.CleanUrl("http://g.msn.com/_0USHP/32?https://u--fr--msn--com.wifi.univ-metz.fr/&&ps=10225&pi=9484&di=81&gt1=&ce=2&cm=wrapper%3Etickler&hl=Search%20Prompt%20shown&su=http://fr.msn.com/","wifi.univ-metz.fr"));
			}
	}	
}
